#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan  4 20:33:37 2017

@author: dawkiny
"""

import math
from pandas import DataFrame as df


class Ellipse:

    def __init__(self, diameter):

        assert isinstance(diameter, tuple)
        assert len(diameter) == 2
        self.diameter = diameter
        self.radius = (int(diameter[0]/2), int(diameter[1]/2))
        self.center = (int(math.floor(diameter[0]/2 + 1)),
                       int(math.floor(diameter[1]/2 + 1)))
        self.angle = 0

    def coordinates(self):

        diameter = self.diameter

        """
        Ellipse equation:
        (x-center_x)^2/radius_x^2 + (y-center_y)^2/radius_y^2 = 1
        x = center_x + radius_x * cos(t)
        y = center_y + radius_y * sin(t)
        """

        # sine, cosine = math.sin(self.angle), math.cos(self.angle)

        # Get Center Coordinates
        center_x, center_y = self.center
        radius_x, radius_y = self.radius

        # Define an Ellipse Function
        coord_list = [(x, y)
                      for x in range(0, diameter[0])
                      for y in range(0, diameter[1])
                      if (pow(x - center_x, 2) / pow(radius_x, 2)) +
                         (pow(y - center_y, 2) / pow(radius_y, 2)) <= 1]

        coords = df(coord_list, columns=['xcoord', 'ycoord'])
        return coords



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jan  5 20:55:26 2017

@author: dawkiny
"""


from pydemia.manufacture.Lot import wafer, displaypanel

#%% wafer

# Create a wafer
alot = wafer(fac_id='Factory1', lot_cd='ABC', end_tm='20161101123456',
             size=(50, 30), unit_cnt=20, pattern=['P', 'F'], p=[.7, .3])

# Check it
alot.groupby(['unit_id', 'val'])['end_tm'].count()
len(alot[alot.unit_id == '01'])

# Visualize it
aplt = wafer.unitplot(alot, columns=['xcoord', 'ycoord'],
                      lot_id='', unit_id='02', val='val', figsize=(12, 9),
                      pattern=['F'], m=['s'], s=[50], c=['red'],
                      a=[.5], shown='y')

# Show it
aplt.show()

# Others
len(alot.groupby('unit_id').groups)
len(alot.groupby('unit_id').groups.items())
alot.groupby('unit_id').groups
alot.groupby('unit_id').groups.items()
alot.groupby('unit_id')['val']


#%% displaypanel

# Create a displaypanel
blot = displaypanel(fac_id='Factory2', lot_cd='BBC', end_tm='20161101123456',
                    size=(60, 20), unit_cnt=5,
                    pattern=['P', 'B', 'F'], p=[.8, 0.15, .05])

# Check it
blot.groupby(['unit_id', 'val'])['end_tm'].count()
len(blot[blot.unit_id == '01'])

# Visualize it
bplt = displaypanel.unitplot(blot, columns=['xcoord', 'ycoord'],
                             lot_id='', unit_id='02', pattern=['B', 'F'],
                             val='val', m=['s', 'o'], s=[50, 50],
                             c=['red', 'blue'], a=[.5, .5], shown='n')

# Show it
bplt.show()





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan  4 20:33:37 2017

@author: dawkiny
"""

import math
from pandas import DataFrame as df

class Ellipse:
    
    def __init__(self, diameter):
        
        assert isinstance(diameter, tuple) == True
        assert len(diameter) == 2
        self.diameter = diameter
        self.radius = (int(diameter[0]/2), int(diameter[1]/2))
        self.center = (int(math.floor(diameter[0]/2 + 1)), int(math.floor(diameter[1]/2 + 1)) )
        self.angle  = 0
        
    def coordinates(self):
        
        diameter = self.diameter
        """                                                                           
        Ellipse equation:                                                             
        (x-center_x)^2/radius_x^2 + (y-center_y)^2/radius_y^2 = 1                     
        x = center_x + radius_x * cos(t)                                              
        y = center_y + radius_y * sin(t)                                              
        """
        
        sine, cosine = math.sin(self.angle), math.cos(self.angle)
        
        # Get Center Coordinates
        center_x, center_y = self.center
        radius_x, radius_y = self.radius
        
        # Define an Ellipse Function
        ellipse_func = lambda x, y: (pow(x - center_x, 2) / pow(radius_x, 2)) + (pow(y - center_y, 2) / pow(radius_y, 2)) <= 1
        coord_list = [(x, y) for x in range(0, diameter[0]) 
                    for y in range(0, diameter[1]) 
                    if (pow(x - center_x, 2) / pow(radius_x, 2)) +
                        (pow(y - center_y, 2) / pow(radius_y, 2)) <= 1]
        
        coords = df(coord_list, columns=['xcoord', 'ycoord'])
        return coords



import pandas as pd
import scipy.stats as st

def chiSquare(data, x=None, y=None, margins=False):


	dataChi = data[[x, y]].dropna()

	crossT = pd.crosstab(dataChi[x], dataChi[y], margins=False)

	print('\n#' + '-'*60 + '#')
	print('#'*20 + ' Chi-Square Test ' + '#'*20 + '\n')
	print('x : ' + x)
	print('y : ' + y)
	print('\n## Contingency Table')
	print('-'*50)
	print(crossT)
	print('-'*50)

	chiSqr, pValue, freeDeg, expectedV = st.chi2_contingency(crossT)
	expectedV = pd.DataFrame(expectedV, index=crossT.index.copy(), columns=crossT.columns.copy())

	msg = '## Test Statistic : {}\n## P-value : {}\n## Degrees of Freedom : {}'

	print('\n\n' + msg.format( chiSqr, pValue, freeDeg ) )
	print('\n' + '## Expected Values')
	print('-'*60)
	print( expectedV )
	print('-'*60)
	print('#' + '-'*60 + '#\n')

	return crossT, chiSqr, pValue, freeDeg, expectedV






#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jul 18 19:39:21 2017

@author: pydemia
"""


import numpy as np
from pandas import DataFrame as df
from datetime import datetime as dt
import itertools as it
import random
import string
import matplotlib.pyplot as plt
from scipy.spatial.distance import pdist
from skimage import draw

from pydemia.math.Geometry import Ellipse


class wafer:
    # Create lotData DataFrame
    def __init__(self, fac_id=None, lot_cd=None, end_tm=None, size=None,
                 unit_cnt=25, f_type='line', p=[.8, .2], pattern=['G', 'B']):

        assert len(size) == 2
        assert len(pattern) == len(p)
        assert sum(p) == 1
        assert isinstance(unit_cnt, int)
        self.fac_id = fac_id
        self.lot_cd = lot_cd
        self.end_tm = end_tm
        self.size = size
        self.unit_cnt = unit_cnt
        self.failrate = p[0]

    def __new__(cls, fac_id=None, lot_cd=None, end_tm=None, size=None,
                unit_cnt=25, f_type='line', p=[.8, .2], pattern=['G', 'B']):

        coords = Ellipse(size).coordinates()

        # xsize, ysize = coords.xcoord.max(), coords.ycoord.max()
        lotDataKey = ['fac_id', 'lot_cd', 'lot_id', 'unit_id',
                      'end_dt', 'end_tm', 'xcoord', 'ycoord', 'val']
        lotData = df(columns=lotDataKey,
                     index=range(0, coords.shape[0] * unit_cnt))

        # Import values to each Field

        # - fab_id, lot_cd
        lotData.fac_id, lotData.lot_cd = fac_id, lot_cd

        # -  alias_lot_id
        id1 = ''.join([random.choice(string.ascii_uppercase +
                                     string.digits) for n in range(1)])
        id2 = ''.join([random.choice(string.digits) for n in range(3)])
        lotData.lot_id = lot_cd + id1 + id2

        # -  wf_id
        unitIter = [list(it.repeat(num, len(coords))) for num in
                    range(1, unit_cnt + 1)]
        lotData.unit_id = list(it.chain.from_iterable(unitIter))
        lotData.unit_id = lotData.unit_id.astype(str).str.zfill(2)

        # -  end_tm, end_dt
        lotData.end_tm = dt.strptime(end_tm, '%Y%m%d%H%M%S')
        lotData.end_dt = lotData.end_tm.map(lambda x: x.date())

        # -  x_coordinate, y_coordinate
        lotData.xcoord = list(coords.xcoord) * unit_cnt
        lotData.ycoord = list(coords.ycoord) * unit_cnt

        # Fail Type Drawing
        lotFailList = []
        for num in range(unit_cnt):

            # f_type = 'Line'
            if f_type == 'line':

                # -  res_val : pattern
                xcoord_range = range(min(coords.xcoord), max(coords.xcoord))
                ycoord_range = range(min(coords.ycoord), max(coords.ycoord))

                dist = 0
                while dist <= 10:
                    adot_x = np.random.choice(xcoord_range)
                    adot_y = np.random.choice(ycoord_range)
                    bdot_x = np.random.choice(xcoord_range)
                    bdot_y = np.random.choice(ycoord_range)

                    adot = adot_x, adot_y
                    bdot = bdot_x, bdot_y
                    dist = pdist([adot, bdot])[0]

                rr, cc = draw.line(adot_x, adot_y, bdot_x, bdot_y)
                patt_coord = [(x, y) for x, y in zip(rr, cc)]
                pattFail = [pattern[1] if (x, y) in patt_coord else pattern[0] for x, y in
                            zip(coords.xcoord, coords.ycoord)]

                # - res_val : add random
                new_fail_num = len(pattFail) * p[1] - pattFail.count(pattern[1])
                new_fail_per = new_fail_num / pattFail.count(pattern[0])
                adjusted_per = 1 - new_fail_per, new_fail_per

                wfFailList = [np.random.choice(pattern, p=adjusted_per) if patt == pattern[0] else patt for patt in
                              pattFail]

                lotFailList += wfFailList

            elif f_type == 'bold_line':

                # -  res_val : pattern
                xcoord_range = range(min(coords.xcoord), max(coords.xcoord))
                ycoord_range = range(min(coords.ycoord), max(coords.ycoord))

                dist = 0
                while dist <= 10:
                    adot_x = np.random.choice(xcoord_range)
                    adot_y = np.random.choice(ycoord_range)
                    bdot_x = np.random.choice(xcoord_range)
                    bdot_y = np.random.choice(ycoord_range)

                    adot = adot_x, adot_y
                    bdot = bdot_x, bdot_y
                    dist = pdist([adot, bdot])[0]

                rr, cc, val = draw.line_aa(adot_x, adot_y, bdot_x, bdot_y)
                patt_coord = [(x, y) for x, y in zip(rr, cc)]
                pattFail = [pattern[1] if (x, y) in patt_coord else pattern[0] for x, y in
                            zip(coords.xcoord, coords.ycoord)]

                # - res_val : add random
                new_fail_num = len(pattFail) * p[1] - pattFail.count(pattern[1])
                new_fail_per = new_fail_num / pattFail.count(pattern[0])
                adjusted_per = 1 - new_fail_per, new_fail_per

                wfFailList = [np.random.choice(pattern, p=adjusted_per) if patt == pattern[0] else patt for patt in
                              pattFail]

                lotFailList += wfFailList

            else:
                pass

        lotData.val = lotFailList

        # Print a message in console
        msg = 'LOT_ID : ' + lot_cd + id1 + id2

        print('-' * len(msg))
        print(msg)
        print('-' * len(msg))
        return lotData

    # lotData Visualization
    def unitplot(data, columns=['xcoord', 'ycoord'], lot_id='', unit_id=None,
                 val='val', figsize=(12, 9), pattern=['B'], m=['s'],
                 s=[100], c=['red'], a=[.5], shown='y'):

        assert len(figsize) == 2
        assert len(columns) == 2
        assert isinstance(lot_id, str)
        assert len(pattern) == len(m) == len(s) == len(c) == len(a)

        # Check if there is a given Wafer ID
        if unit_id is not None:
            pltData = data.loc[data.unit_id == unit_id,
                               [columns[0], columns[1], val]]
        else:
            pltData = data.loc[:, [columns[0], columns[1], val]]
            unit_id = ''

            # Decide if the plot is shown
        #        if shown == 'y':
        #            matplotlib.use('Agg')
        #            plt.ion()
        #
        #        elif shown == 'n':
        #            # Turn interactive plotting off
        #            matplotlib.use('Agg')
        #            plt.ioff()

        canvas = {'pattern': '+', 's': 1000, 'c': 'grey', 'a': .2}

        # Draw plot
        fig = plt.figure(figsize=figsize)
        plt.scatter(pltData.xcoord, pltData.ycoord,
                    marker=canvas['pattern'], s=canvas['s'], c=canvas['c'],
                    alpha=canvas['a'])

        for idx in range(0, len(pattern)):
            plt.scatter(pltData.loc[pltData.val == pattern[idx], 'xcoord'],
                        pltData.loc[pltData.val == pattern[idx], 'ycoord'],
                        marker=m[idx], s=s[idx], c=c[idx], alpha=a[idx])

        fig.suptitle(str(lot_id + unit_id))
        # fig.tight_layout()

        if shown == 'n':
            plt.close(fig)

        return fig


class displaypanel(wafer):
    # Create lotData DataFrame
    def __init__(self, fac_id=None, lot_cd=None, end_tm=None, size=None,
                 unit_cnt=25, pattern=['G', 'B'], p=[.8, .2]):
        assert len(size) == 2
        assert len(pattern) == len(p)
        assert sum(p) == 1
        assert isinstance(unit_cnt, int)

        self.fac_id = fac_id
        self.lot_cd = lot_cd
        self.end_tm = end_tm
        self.size = size
        self.unit_cnt = unit_cnt
        self.failrate = p[0]

    def __new__(cls, fac_id=None, lot_cd=None, end_tm=None, size=None,
                unit_cnt=25, pattern=['G', 'B'], p=[.8, .2]):
        xsize, ysize = size
        lotDataKey = ['fac_id', 'lot_cd', 'lot_id', 'unit_id',
                      'end_dt', 'end_tm', 'xcoord', 'ycoord', 'val']

        lotData = df(columns=lotDataKey,
                     index=range(0, xsize * ysize * unit_cnt))

        # Import values to each Field

        # -  fab_id, lot_cd
        lotData.fac_id, lotData.lot_cd = fac_id, lot_cd

        # -  alias_lot_id
        id1 = ''.join([random.choice(string.ascii_uppercase +
                                     string.digits) for n in range(1)])
        id2 = ''.join([random.choice(string.digits) for n in range(3)])
        lotData.lot_id = lot_cd + id1 + id2

        # -  pn_id
        unitIter = [list(it.repeat(num, xsize * ysize)) for num
                    in range(1, unit_cnt + 1)]
        lotData.unit_id = list(it.chain.from_iterable(unitIter))
        lotData.unit_id = lotData.unit_id.astype(str).str.zfill(2)

        # -  end_tm, end_dt
        lotData.end_tm = dt.strptime(end_tm, '%Y%m%d%H%M%S')
        lotData.end_dt = lotData.end_tm.map(lambda x: x.date())

        # -  x_coordinate, y_coordinate
        coords = tuple(it.product(list(range(1, xsize + 1)),
                                  list(range(1, ysize + 1))))
        lotData.xcoord, lotData.ycoord = zip(*coords * unit_cnt)

        # -  res_val
        failList = [np.random.choice(pattern,
                                     size=len(coords),
                                     p=p) for num in range(1, unit_cnt + 1)]
        lotData.val = list(it.chain.from_iterable(failList))

        # Print a message in console
        msg = 'LOT_ID : ' + lot_cd + id1 + id2
        print('-' * len(msg))
        print(msg)
        print('-' * len(msg))
        return lotData


if __name__ == '__main__':

    alot = wafer(fac_id='Factory1', lot_cd='ABC', end_tm='20161101123456',
                 size=(55, 35), unit_cnt=20, p=[.85, .15], f_type='bold_line')
    aplt = wafer.unitplot(alot, columns=['xcoord', 'ycoord'],
                          lot_id='', unit_id='02', val='val', figsize=(11, 10),
                          pattern=['F'], m=['s'], s=[250], c=['red'], a=[.8], shown='y')
    aplt.show()
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jul 18 19:39:21 2017

@author: pydemia
"""


import numpy as np
import pandas as pd
from datetime import datetime as dt
import itertools as it
import random
import string
import matplotlib.pyplot as plt
from scipy.spatial.distance import pdist
from skimage import draw

from pydemia.math.Geometry import Ellipse


class wafer(object):

    # Create lotData DataFrame
    def __init__(self, fac_id=None, lot_cd=None, end_tm=None, size=None,
                 unit_cnt=25, f_type='line', y_val=.8,
                 *args, **kwargs):

        assert len(size) == 2
        assert y_val < 1
        assert isinstance(unit_cnt, int)
        self.fac_id = fac_id
        self.lot_cd = lot_cd
        self.end_tm = end_tm
        self.size = size
        self.unit_cnt = unit_cnt
        self.f_type=f_type
        self.y_val = y_val
        self.f_val = 1 - y_val
        self.pattern = ['G', 'B']

        #self.lotData = super(wafer, cls).__new__(cls, *args, **kwargs)
        self.lotData = self._create()

#    def __new__(cls, *args, **kwargs):
#
#        obj = super(wafer, cls).__create__(lot, *args, **kwargs)
#        obj.dd = obj
#        return obj

    def _create(self):
        
        fac_id = self.fac_id
        lot_cd = self.lot_cd
        end_tm = self.end_tm
        size = self.size
        unit_cnt = self.unit_cnt
        f_type=self.f_type
        y_val = self.y_val
        f_val = self.f_val
        pattern = self.pattern
        
        
        
        assert len(size) == 2
        assert y_val < 1
        assert isinstance(unit_cnt, int)

        f_val = 1 - y_val
        pattern = ['G', 'B']
        good_mark = pattern[0]  # 'G'
        fail_mark = pattern[1]  # 'B'

        # Adjust Egde Coordinates
        coords = Ellipse(size).coordinates()
        
        x_cnt = coords.groupby('xcoord')['ycoord'].nunique()
        y_cnt = coords.groupby('ycoord')['xcoord'].nunique()
        
        x_except = x_cnt[x_cnt < 3].index.values[0]
        y_except = y_cnt[y_cnt < 3].index.values[0]

        coords = coords[(coords.xcoord!=x_except) & (coords.ycoord!=y_except)]
        

        # xsize, ysize = coords.xcoord.max(), coords.ycoord.max()
        lotDataKey = ['fac_id', 'lot_cd', 'lot_id', 'unit_id',
                      'end_dt', 'end_tm', 'xcoord', 'ycoord', 'val']
        lotData = pd.DataFrame(columns=lotDataKey,
                     index=range(0, coords.shape[0] * unit_cnt))

        # Import values to each Field

        # - fab_id, lot_cd
        lotData.fac_id, lotData.lot_cd = fac_id, lot_cd

        # -  alias_lot_id
        id1 = ''.join([random.choice(string.ascii_uppercase +
                                     string.digits) for n in range(1)])
        id2 = ''.join([random.choice(string.digits) for n in range(3)])
        lotData.lot_id = lot_cd + id1 + id2

        # -  wf_id
        unitIter = [list(it.repeat(num, len(coords))) for num in
                    range(1, unit_cnt + 1)]
        lotData.unit_id = list(it.chain.from_iterable(unitIter))
        lotData.unit_id = lotData.unit_id.astype(str).str.zfill(2)

        # -  end_tm, end_dt
        lotData.end_tm = dt.strptime(end_tm, '%Y%m%d%H%M%S')
        lotData.end_dt = lotData.end_tm.map(lambda x: x.date())

        # -  x_coordinate, y_coordinate
        lotData.xcoord = list(coords.xcoord) * unit_cnt
        lotData.ycoord = list(coords.ycoord) * unit_cnt

        # Fail Type Drawing
        lotFailList = []
        for num in range(unit_cnt):

            # f_type = 'Line''line'
            if f_type == 'line':

                # -  res_val : pattern
                xcoord_range = range(min(coords.xcoord), max(coords.xcoord))
                ycoord_range = range(min(coords.ycoord), max(coords.ycoord))

                dist = 0
                while dist <= 15:
                    adot_x = np.random.choice(xcoord_range)
                    adot_y = np.random.choice(ycoord_range) 
                    bdot_x = np.random.choice(xcoord_range)
                    bdot_y = np.random.choice(ycoord_range)

                    adot = adot_x, adot_y
                    bdot = bdot_x, bdot_y
                    dist = pdist([adot, bdot])[0]

                rr, cc = draw.line(adot_x, adot_y, bdot_x, bdot_y)
                patt_coord = [(x, y) for x, y in zip(rr, cc)]
                pattFail = [fail_mark if (x, y) in patt_coord else good_mark for x, y in
                            zip(coords.xcoord, coords.ycoord)]

                # - res_val : add random
                new_fail_num = len(pattFail) * f_val - pattFail.count(fail_mark)
                
                if new_fail_num < 0:
                    pass
                else:
                    new_fail_per = new_fail_num / pattFail.count(good_mark)
                    adjusted_per = 1 - new_fail_per, new_fail_per
    
                    wfFailList = [np.random.choice(pattern, p=adjusted_per) if patt == good_mark else patt for patt in
                                  pattFail]

                lotFailList += wfFailList

            elif f_type == 'bold_line':

                # -  res_val : pattern
                xcoord_range = range(min(coords.xcoord), max(coords.xcoord))
                ycoord_range = range(min(coords.ycoord), max(coords.ycoord))

                dist = 0
                while dist <= 15:
                    adot_x = np.random.choice(xcoord_range)
                    adot_y = np.random.choice(ycoord_range)
                    bdot_x = np.random.choice(xcoord_range)
                    bdot_y = np.random.choice(ycoord_range)

                    adot = adot_x, adot_y
                    bdot = bdot_x, bdot_y
                    dist = pdist([adot, bdot])[0]

                rr, cc, val = draw.line_aa(adot_x, adot_y, bdot_x, bdot_y)
                patt_coord = [(x, y) for x, y in zip(rr, cc)]
                pattFail = [fail_mark if (x, y) in patt_coord else good_mark for x, y in
                            zip(coords.xcoord, coords.ycoord)]

                # - res_val : add random
                new_fail_num = len(pattFail) * f_val - pattFail.count(fail_mark)
                
                if new_fail_num < 0:
                    pass
                else:
                    new_fail_per = new_fail_num / pattFail.count(good_mark)
                    adjusted_per = 1 - new_fail_per, new_fail_per
    
                    wfFailList = [np.random.choice(pattern, p=adjusted_per) if patt == good_mark else patt for patt in
                                  pattFail]

                lotFailList += wfFailList

            elif f_type == 'none':

                wfFailList = [np.random.choice(pattern, size=len(coords), p=(y_val, f_val)) for num in range(1, unit_cnt + 1)]
                lotFailList = list(it.chain.from_iterable(wfFailList))

        lotData.val = lotFailList

        # Print a message in console
        msg1 = ' LOT_ID : {}'.format(lot_cd + id1 + id2)
        msg2 = ' WF_CNT : {}'.format(unit_cnt)

        print('-' * (len(msg1) + 2))
        print(msg1)
        print(msg2)
        print('-' * (len(msg1) + 2))

        return lotData

    # lotData Visualization
    def unitplot(self, columns=['xcoord', 'ycoord'], lot_id=None, unit_id=None,
                 val='val', figsize=(11, 10), pattern=['B'], m=['s'],
                 s=[120], c=['black'], a=[1.], background=True,
                 title_off=False, tight_on=False, axis_off=False,
                 shown=True):

        data = self.lotData
        assert len(figsize) == 2
        assert len(columns) == 2
        assert len(pattern) == len(m) == len(s) == len(c) == len(a)

        # Lot ID
        if lot_id is None:    
            lot_id = data.lot_id.unique()[0]

        # Check if there is a given Wafer ID
        if unit_id is not None:
            pltData = data.loc[data.unit_id == unit_id,
                               [columns[0], columns[1], val]]
        else:
            pltData = data.loc[:, [columns[0], columns[1], val]]
            unit_id = ''

            # Decide if the plot is shown
        #        if shown == 'y':
        #            matplotlib.use('Agg')
        #            plt.ion()
        #
        #        elif shown == 'n':
        #            # Turn interactive plotting off
        #            matplotlib.use('Agg')
        #            plt.4ioff()

        canvas = {'pattern': '+',
                  's': 1000,
                  'c': 'grey',
                  'a': .2}

        # Draw plot
        fig = plt.figure(figsize=figsize)
        
        if background == True:
            plt.scatter(pltData.xcoord, pltData.ycoord,
                        marker='s', s=120, alpha=1.,
                        facecolors='white', edgecolors='lightgrey')

        for idx in range(0, len(pattern)):
            plt.scatter(pltData.loc[pltData.val == pattern[idx], 'xcoord'],
                        pltData.loc[pltData.val == pattern[idx], 'ycoord'],
                        marker=m[idx], s=s[idx], c=c[idx], alpha=a[idx])

        if title_off == False:
            fig.suptitle('{}, {}'.format(lot_id, unit_id))

        if tight_on == True:
            fig.tight_layout()

        if axis_off == True:
            plt.axis('off')

        if shown == False:
            plt.close(fig)

        return fig


if __name__ == '__main__':
    
    alot = wafer(fac_id='Factory1', lot_cd='ABC', end_tm='20161101123456',
                 size=(55, 45), unit_cnt=20, y_val=.8, f_type='line')
    alot.lotData
    alot.unitplot(unit_id='02',figsize=(11, 10),
                  m=['s'], s=[120], c=['black'], a=[1.], axis_off=True)





from statsmodels.stats.outliers_influence import variance_inflation_factor as sv

def vif(data, thresh=5.0):

	dropped = True
	
	while dropped == True :
		variables = list(range(data.shape[1])

		dropped = False
		
		vifTable = [sv(data[variables].values, col) for col in range(data[variables].shape[1])]

		maxLoc = vifTable.index(max(vifTable))

		if max(vifTable) >= thresh:

			print('Dropping \'' + data[variables].columns[maxLoc] + '\' at index: ' + str(maxLoc))

			data = data.drop(data.columns[maxLoc], axis=1)
			del variables[maxLoc]

			dropped = True

	print('Remaining Variables')
	print(data.columns[variables])

	return data[variables].columns, data[variables]



import pandas as pd
from pandas import DataFrame as df
from datetime import datetime

import psycopg2 as pg
import sqlalchemy as sa
# import ibm_db
# import ibm_db_sa
import cx_Oracle as co
import pymysql


def postgreSQL(query, h=None, port=None, db=None, u=None, p=None):

    print('Using PostgreSQL')

    # DB Connection
    conn = pg.connect(host=h, port=str(port), user=u, password=p)
    start_tm = datetime.now()
    print(' Start :', str(start_tm))

    # Get a DataFrame
    query_result = pd.read_sql(query, conn)

    # Cloase Connection
    end_tm = datetime.now()
    print(' Finish :', str(end_tm))
    print('Elapsed :', str(end_tm - start_tm))
    conn.close()

    return query_result


def db2SQL(query, h=None, port=None, db=None, u=None, p=None):

    print('Using IBM DB2')

    # DB Connection
    connStr = 'ibm_db_sa://{}:{}@{}:{}/{}'
    engine = sa.create_engine(connStr.format(u, p, h, str(port), db))
    conn = engine.connect()
    start_tm = datetime.now()
    print(' Start :', str(start_tm))

    # Get a DataFrame
    execonn = engine.execute(query)

    query_result = df(execonn.fetchall())
    query_result.columns = execonn.keys()

    # Close Connection
    end_tm = datetime.now()
    print(' Finish :', str(end_tm))
    print('Elapsed :', str(end_tm - start_tm))
    conn.close()

    return query_result


def oracleSQL(query, h=None, port=None, db=None, u=None, p=None):

    print('Using OracleDB')

    # DB Connection
    dnsStr = co.makedsn(h, str(port), db)
    dnsStr = dnsStr.replace('SID', 'SERVICE_NAME')
    conn = co.connect(u, p, dnsStr)
    start_tm = datetime.now()

    # Get a DataFrame
    query_result = pd.read_sql(query, conn)

    # Close Connection
    end_tm = datetime.now()
    print(' Finish :', str(end_tm))
    print('Elapsed :', str(end_tm - start_tm))
    conn.close()

    return query_result


def mariaDB(query, h=None, port=None, db=None, u=None, p=None):

    print('Using MariaDB')

    # DB Connection
    conn = pymysql.connect(host=h, port=p, user=u, password=p, database=db)
    start_tm = datetime.now()
    print(' Start :', str(start_tm))

    # Get a DataFrame
    query_result = pd.read_sql(query, conn)

    # Close Connection
    end_tm = datetime.now()
    print(' Finish :', str(end_tm))
    print('Elapsed :', str(end_tm - start_tm))
    conn.close()

    return query_result



from setuptools import setup, find_packages


long_desc = """
This is made for some specific environment.
This contains some db connection tools & analysis tools. 
"""

setup(name='pydemia',
	version='0.0.96',
	description='Useful tools for Analysis',
	long_description=long_desc,
	url='http://github.com/pydemia/pydemia',
	author='Young Ju Kim',
	author_email='pydemia@gmail.com',
	license='MIT License',
	classifiers=[
		# How Mature: 3 - Alpha, 4 - Beta, 5 - Production/Stable
		'Development Status :: 3 - Alpha',
		'Programming Language :: Python :: 3.5'
		],
	packages=find_packages(exclude=['contrib', 'docs', 'tests']),
	install_requires=[
			  'scikit-image', 'pandas', 'numpy', 'scipy', 'statsmodels', 
				'pymysql', 'psycopg2', 'sqlalchemy',
				'ibm_db_sa' 
				#'cx_Oracle'
					],
	zip_safe=False)

